"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const lua = require("../../../LuaAST");
const utils_1 = require("../../../utils");
const export_1 = require("../../utils/export");
const lua_ast_1 = require("../../utils/lua-ast");
const lualib_1 = require("../../utils/lualib");
const utils_2 = require("./utils");
function createClassSetup(context, statement, className, localClassName, classNameText, extendsType) {
    const result = [];
    // __TS__Class()
    const classInitializer = lualib_1.transformLuaLibFunction(context, lualib_1.LuaLibFeature.Class, statement);
    const defaultExportLeftHandSide = export_1.hasDefaultExportModifier(statement)
        ? lua.createTableIndexExpression(lua_ast_1.createExportsIdentifier(), export_1.createDefaultExportStringLiteral(statement))
        : undefined;
    // [____exports.]className = __TS__Class()
    if (defaultExportLeftHandSide) {
        result.push(lua.createAssignmentStatement(defaultExportLeftHandSide, classInitializer, statement));
    }
    else {
        result.push(...lua_ast_1.createLocalOrExportedOrGlobalDeclaration(context, className, classInitializer, statement));
    }
    if (defaultExportLeftHandSide) {
        // local localClassName = ____exports.default
        result.push(lua.createVariableDeclarationStatement(localClassName, defaultExportLeftHandSide));
    }
    else {
        const exportScope = export_1.getIdentifierExportScope(context, className);
        if (exportScope) {
            // local localClassName = ____exports.className
            result.push(lua.createVariableDeclarationStatement(localClassName, export_1.createExportedIdentifier(context, lua.cloneIdentifier(className), exportScope)));
        }
    }
    // localClassName.name = className
    result.push(lua.createAssignmentStatement(lua.createTableIndexExpression(lua.cloneIdentifier(localClassName), lua.createStringLiteral("name")), lua.createStringLiteral(classNameText), statement));
    if (extendsType) {
        const extendedNode = utils_2.getExtendedNode(context, statement);
        utils_1.assert(extendedNode);
        result.push(lua.createExpressionStatement(lualib_1.transformLuaLibFunction(context, lualib_1.LuaLibFeature.ClassExtends, utils_2.getExtendsClause(statement), lua.cloneIdentifier(localClassName), context.transformExpression(extendedNode.expression))));
    }
    return result;
}
exports.createClassSetup = createClassSetup;
//# sourceMappingURL=setup.js.map